@@ -15,7 +15,7 @@ def prune(app):
     for env_name, config in environments.items():
         environment_type = config['type']
         if environment_type not in environment_types:
-            app.abort(f'Environment `{app.env}` has unknown type: {environment_type}')
+            app.abort(f'Environment `{env_name}` has unknown type: {environment_type}')
 
         data_dir = app.get_env_directory(environment_type)
 
@@ -35,5 +35,6 @@ def prune(app):
         except Exception:
             continue
 
-        if environment.exists():
-            environment.remove()
+        if environment.exists() or environment.build_environment_exists():
+            with app.status_waiting(f'Removing environment: {env_name}'):
+                environment.remove()
