@@ -6,7 +6,7 @@ from os.path import isabs
 from hatch.env.plugin.interface import EnvironmentInterface
 from hatch.utils.fs import Path
 from hatch.utils.shells import ShellManager
-from hatch.venv.core import TempVirtualEnv, VirtualEnv
+from hatch.venv.core import VirtualEnv
 
 
 class VirtualEnvironment(EnvironmentInterface):
@@ -15,27 +15,42 @@ class VirtualEnvironment(EnvironmentInterface):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
 
-        project_name = self.metadata.core.name
+        project_name = self.metadata.name
+        venv_name = project_name if self.name == 'default' else self.name
 
+        # Always compute the app path for build environments
         hashed_root = sha256(str(self.root).encode('utf-8')).digest()
         checksum = urlsafe_b64encode(hashed_root).decode('utf-8')[:8]
-        self.storage_path = self.data_directory / f'{project_name}-{checksum}'
+        app_storage_path = self.data_directory / project_name / checksum
+        app_virtual_env_path = app_storage_path / venv_name
 
-        chosen_directory = self.get_env_var_option('path')
+        # Explicit path
+        chosen_directory = self.get_env_var_option('path') or self.config.get('path', '')
         if chosen_directory:
-            self.virtual_env_path = Path(chosen_directory) if isabs(chosen_directory) else self.root / chosen_directory
+            self.storage_path = app_storage_path
+            self.virtual_env_path = (
+                Path(chosen_directory) if isabs(chosen_directory) else (self.root / chosen_directory).resolve()
+            )
+        # Conditions requiring a flat structure
+        elif self.root in self.data_directory.parents or self.data_directory == Path.home() / '.virtualenvs':
+            self.storage_path = self.data_directory
+            self.virtual_env_path = self.storage_path / venv_name
+        # Otherwise the standard app path
         else:
-            directory = project_name if self.name == 'default' else self.name
-            self.virtual_env_path = self.storage_path / directory
+            self.storage_path = app_storage_path
+            self.virtual_env_path = app_virtual_env_path
 
         self.virtual_env = VirtualEnv(self.virtual_env_path, self.platform, self.verbosity)
+        self.build_virtual_env = VirtualEnv(
+            app_virtual_env_path.parent / f'{app_virtual_env_path.name}-build', self.platform, self.verbosity
+        )
         self.shells = ShellManager(self)
 
         self._parent_python = None
 
     @staticmethod
     def get_option_types() -> dict:
-        return {'system-packages': bool}
+        return {'system-packages': bool, 'path': bool}
 
     def activate(self):
         self.virtual_env.activate()
@@ -47,14 +62,30 @@ class VirtualEnvironment(EnvironmentInterface):
         return self.virtual_env_path
 
     def create(self):
+        if self.root in self.storage_path.parents:
+            # Although it would be nice to support Mercurial, only Git supports multiple ignore files. See:
+            # https://github.com/pytest-dev/pytest/issues/3286#issuecomment-421439197
+            vcs_ignore_file = self.storage_path / '.gitignore'
+            if not vcs_ignore_file.is_file():
+                vcs_ignore_file.ensure_parent_dir_exists()
+                vcs_ignore_file.write_text(
+                    """\
+# This file was automatically created by Hatch
+*
+"""
+                )
+
         self.virtual_env.create(self.parent_python, allow_system_packages=self.config.get('system-packages', False))
 
     def remove(self):
         self.virtual_env.remove()
+        self.build_virtual_env.remove()
 
         # Clean up root directory of all virtual environments belonging to the project
-        if self.storage_path.is_dir() and not any(self.storage_path.iterdir()):
-            self.storage_path.remove()
+        if self.storage_path != Path.home() / '.virtualenvs' and self.storage_path.is_dir():
+            entries = [entry.name for entry in self.storage_path.iterdir()]
+            if not entries or (entries == ['.gitignore'] and self.root in self.storage_path.parents):
+                self.storage_path.remove()
 
     def exists(self):
         return self.virtual_env.exists()
@@ -86,11 +117,26 @@ class VirtualEnvironment(EnvironmentInterface):
 
     @contextmanager
     def build_environment(self, dependencies):
-        with self.get_env_vars(), TempVirtualEnv(self.parent_python, self.platform, self.verbosity):
-            self.platform.check_command(self.construct_pip_install_command(dependencies))
+        from packaging.requirements import Requirement
+
+        from hatchling.dep.core import dependencies_in_sync
+
+        if not self.build_environment_exists():
+            self.build_virtual_env.create(self.parent_python)
+
+        with self.get_env_vars(), self.build_virtual_env:
+            if not dependencies_in_sync(
+                [Requirement(d) for d in dependencies],
+                sys_path=self.build_virtual_env.sys_path,
+                environment=self.build_virtual_env.environment,
+            ):
+                self.platform.check_command(self.construct_pip_install_command(dependencies))
 
             yield
 
+    def build_environment_exists(self):
+        return self.build_virtual_env.exists()
+
     @contextmanager
     def command_context(self):
         with self.safe_activation():
