@@ -1,7 +1,9 @@
-try:
+import sys
+
+if sys.version_info >= (3, 11):
     import tomllib
-except ImportError:
-    import tomli as tomllib  # type: ignore
+else:
+    import tomli as tomllib
 
 
 def load_toml_data(data):
@@ -9,5 +11,5 @@ def load_toml_data(data):
 
 
 def load_toml_file(path):
-    with open(path, 'r', encoding='utf-8') as f:
+    with open(path, encoding='utf-8') as f:
         return tomllib.loads(f.read())
