@@ -1608,7 +1608,7 @@ class TestFileSelectionDefaults:
     def test_global_exclude(self, isolation):
         builder = Builder(str(isolation))
 
-        assert builder.config.default_global_exclude() == ['*.py[cdo]', '.git', '__pycache__']
+        assert builder.config.default_global_exclude() == ['*.py[cdo]', '/dist']
 
 
 class TestPatternInclude:
@@ -1621,7 +1621,7 @@ class TestPatternInclude:
         config = {'tool': {'hatch': {'build': {'include': ['foo']}}}}
         builder = Builder(str(isolation), config=config)
 
-        assert isinstance(builder.config.include_spec, pathspec.PathSpec)
+        assert isinstance(builder.config.include_spec, pathspec.GitIgnoreSpec)
 
     def test_global_invalid_type(self, isolation):
         config = {'tool': {'hatch': {'build': {'include': ''}}}}
@@ -1737,8 +1737,8 @@ class TestPatternExclude:
 
         builder = Builder(str(isolation))
 
-        assert isinstance(builder.config.exclude_spec, pathspec.PathSpec)
-        assert builder.config.exclude_spec.match_file(f'.git{separator}file.py')
+        assert isinstance(builder.config.exclude_spec, pathspec.GitIgnoreSpec)
+        assert builder.config.exclude_spec.match_file(f'dist{separator}file.py')
 
     def test_global_invalid_type(self, isolation):
         config = {'tool': {'hatch': {'build': {'exclude': ''}}}}
@@ -1902,7 +1902,7 @@ class TestPatternArtifacts:
         config = {'tool': {'hatch': {'build': {'artifacts': ['foo']}}}}
         builder = Builder(str(isolation), config=config)
 
-        assert isinstance(builder.config.artifact_spec, pathspec.PathSpec)
+        assert isinstance(builder.config.artifact_spec, pathspec.GitIgnoreSpec)
 
     def test_global_invalid_type(self, isolation):
         config = {'tool': {'hatch': {'build': {'artifacts': ''}}}}
