@@ -674,9 +674,9 @@ class TestCoreMetadataV21:
             """
         )
 
-    def test_all(self, constructor, isolation, helpers):
+    def test_all(self, constructor, isolation, helpers, temp_dir):
         metadata = ProjectMetadata(
-            str(isolation),
+            str(temp_dir),
             None,
             {
                 'project': {
@@ -697,12 +697,16 @@ class TestCoreMetadataV21:
                     'optional-dependencies': {
                         'feature2': ['foo==1; python_version < "3"', 'bar==5'],
                         'feature1': ['foo==1', 'bar==5; python_version < "3"'],
+                        'feature3': ['baz @ file:///path/to/project'],
                     },
                     'readme': {'content-type': 'text/markdown', 'text': 'test content\n'},
-                }
+                },
+                'tool': {'hatch': {'metadata': {'allow-direct-references': True}}},
             },
         )
 
+        (temp_dir / 'LICENSE.txt').touch()
+
         assert constructor(metadata) == helpers.dedent(
             """
             Metadata-Version: 2.1
@@ -715,6 +719,7 @@ class TestCoreMetadataV21:
             Maintainer-email: foo <bar@domain>
             License: foo
                     bar
+            License-File: LICENSE.txt
             Keywords: bar,foo
             Classifier: Programming Language :: Python :: 3.9
             Classifier: Programming Language :: Python :: 3.11
@@ -727,6 +732,8 @@ class TestCoreMetadataV21:
             Provides-Extra: feature2
             Requires-Dist: bar==5; extra == 'feature2'
             Requires-Dist: foo==1; python_version < '3' and extra == 'feature2'
+            Provides-Extra: feature3
+            Requires-Dist: baz@ file:///path/to/project ; extra == 'feature3'
             Description-Content-Type: text/markdown
 
             test content
@@ -1073,9 +1080,9 @@ class TestCoreMetadataV22:
             """
         )
 
-    def test_all(self, constructor, isolation, helpers):
+    def test_all(self, constructor, isolation, helpers, temp_dir):
         metadata = ProjectMetadata(
-            str(isolation),
+            str(temp_dir),
             None,
             {
                 'project': {
@@ -1096,12 +1103,16 @@ class TestCoreMetadataV22:
                     'optional-dependencies': {
                         'feature2': ['foo==1; python_version < "3"', 'bar==5'],
                         'feature1': ['foo==1', 'bar==5; python_version < "3"'],
+                        'feature3': ['baz @ file:///path/to/project'],
                     },
                     'readme': {'content-type': 'text/markdown', 'text': 'test content\n'},
-                }
+                },
+                'tool': {'hatch': {'metadata': {'allow-direct-references': True}}},
             },
         )
 
+        (temp_dir / 'LICENSE.txt').touch()
+
         assert constructor(metadata) == helpers.dedent(
             """
             Metadata-Version: 2.2
@@ -1114,6 +1125,7 @@ class TestCoreMetadataV22:
             Maintainer-email: foo <bar@domain>
             License: foo
                     bar
+            License-File: LICENSE.txt
             Keywords: bar,foo
             Classifier: Programming Language :: Python :: 3.9
             Classifier: Programming Language :: Python :: 3.11
@@ -1126,6 +1138,8 @@ class TestCoreMetadataV22:
             Provides-Extra: feature2
             Requires-Dist: bar==5; extra == 'feature2'
             Requires-Dist: foo==1; python_version < '3' and extra == 'feature2'
+            Provides-Extra: feature3
+            Requires-Dist: baz@ file:///path/to/project ; extra == 'feature3'
             Description-Content-Type: text/markdown
 
             test content
@@ -1519,9 +1533,11 @@ class TestCoreMetadataV23:
                     'optional-dependencies': {
                         'feature2': ['foo==1; python_version < "3"', 'bar==5'],
                         'feature1': ['foo==1', 'bar==5; python_version < "3"'],
+                        'feature3': ['baz @ file:///path/to/project'],
                     },
                     'readme': {'content-type': 'text/markdown', 'text': 'test content\n'},
-                }
+                },
+                'tool': {'hatch': {'metadata': {'allow-direct-references': True}}},
             },
         )
 
@@ -1555,6 +1571,8 @@ class TestCoreMetadataV23:
             Provides-Extra: feature2
             Requires-Dist: bar==5; extra == 'feature2'
             Requires-Dist: foo==1; python_version < '3' and extra == 'feature2'
+            Provides-Extra: feature3
+            Requires-Dist: baz@ file:///path/to/project ; extra == 'feature3'
             Description-Content-Type: text/markdown
 
             test content
