@@ -4,6 +4,8 @@ from hatch.config.constants import AppEnvVars, ConfigEnvVars
 from hatch.project.core import Project
 from hatch.utils.structures import EnvVars
 from hatch.venv.core import VirtualEnv
+from hatchling.utils.constants import DEFAULT_BUILD_SCRIPT
+from hatchling.utils.fs import path_to_uri
 
 
 def test_undefined(hatch, helpers, temp_dir, config_file):
@@ -107,16 +109,14 @@ def test_new(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -171,16 +171,14 @@ def test_new_selected_python(hatch, helpers, temp_dir, config_file, python_on_pa
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -233,16 +231,14 @@ def test_selected_absolute_directory(hatch, helpers, temp_dir, config_file):
 
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -252,7 +248,7 @@ def test_selected_absolute_directory(hatch, helpers, temp_dir, config_file):
     assert env_path.name == 'test'
 
 
-def test_env_var_absolute_directory(hatch, helpers, temp_dir, config_file):
+def test_option_absolute_directory(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.model.dirs.env = {'virtual': '$VENVS_DIR'}
     config_file.save()
@@ -271,7 +267,54 @@ def test_env_var_absolute_directory(hatch, helpers, temp_dir, config_file):
     project = Project(project_path)
     assert project.config.envs == {'default': {'type': 'virtual'}}
     helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
-    helpers.update_project_environment(project, 'test', {})
+    helpers.update_project_environment(project, 'test', {'path': str(env_path)})
+
+    with project_path.as_cwd({'VENVS_DIR': str(env_data_path)}):
+        result = hatch('env', 'create', 'test')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Creating environment: test
+        Checking dependencies
+        """
+    )
+
+    project = Project(project_path)
+    assert project.config.envs == {
+        'default': {'type': 'virtual', 'skip-install': True},
+        'test': {'type': 'virtual', 'skip-install': True, 'path': str(env_path)},
+    }
+    assert project.raw_config['tool']['hatch']['envs'] == {
+        'default': {'type': 'virtual', 'skip-install': True},
+        'test': {'path': str(env_path)},
+    }
+
+    assert not env_data_path.is_dir()
+    assert env_path.is_dir()
+
+
+def test_env_var_absolute_directory(hatch, helpers, temp_dir, config_file):
+    config_file.model.template.plugins['default']['tests'] = False
+    config_file.model.dirs.env = {'virtual': '$VENVS_DIR'}
+    config_file.save()
+
+    project_name = 'My.App'
+
+    with temp_dir.as_cwd():
+        result = hatch('new', project_name)
+
+    assert result.exit_code == 0, result.output
+
+    project_path = temp_dir / 'my-app'
+    env_data_path = temp_dir / '.venvs'
+    env_path = temp_dir / 'foo'
+    env_path_overridden = temp_dir / 'bar'
+
+    project = Project(project_path)
+    assert project.config.envs == {'default': {'type': 'virtual'}}
+    helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
+    helpers.update_project_environment(project, 'test', {'path': str(env_path_overridden)})
 
     with project_path.as_cwd({'VENVS_DIR': str(env_data_path), 'HATCH_ENV_TYPE_VIRTUAL_PATH': str(env_path)}):
         result = hatch('env', 'create', 'test')
@@ -287,11 +330,11 @@ def test_env_var_absolute_directory(hatch, helpers, temp_dir, config_file):
     project = Project(project_path)
     assert project.config.envs == {
         'default': {'type': 'virtual', 'skip-install': True},
-        'test': {'type': 'virtual', 'skip-install': True},
+        'test': {'type': 'virtual', 'skip-install': True, 'path': str(env_path_overridden)},
     }
     assert project.raw_config['tool']['hatch']['envs'] == {
         'default': {'type': 'virtual', 'skip-install': True},
-        'test': {},
+        'test': {'path': str(env_path_overridden)},
     }
 
     assert not env_data_path.is_dir()
@@ -312,12 +355,75 @@ def test_selected_local_directory(hatch, helpers, temp_dir, config_file):
 
     project_path = temp_dir / 'my-app'
 
+    project = Project(project_path)
+    helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
+    helpers.update_project_environment(project, 'test', {'matrix': [{'version': ['9000', '42']}]})
+
+    with project_path.as_cwd({'VENVS_DIR': '.hatch'}):
+        result = hatch('env', 'create')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Creating environment: default
+        Checking dependencies
+        """
+    )
+
+    with project_path.as_cwd({'VENVS_DIR': '.hatch'}):
+        result = hatch('env', 'create', 'test')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Creating environment: test.9000
+        Checking dependencies
+        Creating environment: test.42
+        Checking dependencies
+        """
+    )
+
+    project = Project(project_path)
+    assert project.config.envs == {
+        'default': {'type': 'virtual', 'skip-install': True},
+        'test.9000': {'type': 'virtual', 'skip-install': True},
+        'test.42': {'type': 'virtual', 'skip-install': True},
+    }
+    assert project.raw_config['tool']['hatch']['envs'] == {
+        'default': {'type': 'virtual', 'skip-install': True},
+        'test': {'matrix': [{'version': ['9000', '42']}]},
+    }
+
+    env_data_path = project_path / '.hatch'
+    assert env_data_path.is_dir()
+
+    env_dirs = list(env_data_path.iterdir())
+    assert len(env_dirs) == 4
+
+    assert sorted(entry.name for entry in env_dirs) == ['.gitignore', 'my-app', 'test.42', 'test.9000']
+
+
+def test_option_local_directory(hatch, helpers, temp_dir, config_file):
+    config_file.model.template.plugins['default']['tests'] = False
+    config_file.model.dirs.env = {'virtual': '$VENVS_DIR'}
+    config_file.save()
+
+    project_name = 'My.App'
+    env_data_path = temp_dir / '.venvs'
+
+    with temp_dir.as_cwd():
+        result = hatch('new', project_name)
+
+    assert result.exit_code == 0, result.output
+
+    project_path = temp_dir / 'my-app'
+
     project = Project(project_path)
     assert project.config.envs == {'default': {'type': 'virtual'}}
     helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
-    helpers.update_project_environment(project, 'test', {})
+    helpers.update_project_environment(project, 'test', {'path': '.venv'})
 
-    with project_path.as_cwd({'VENVS_DIR': '.venvs'}):
+    with project_path.as_cwd({'VENVS_DIR': str(env_data_path)}):
         result = hatch('env', 'create', 'test')
 
     assert result.exit_code == 0, result.output
@@ -331,33 +437,14 @@ def test_selected_local_directory(hatch, helpers, temp_dir, config_file):
     project = Project(project_path)
     assert project.config.envs == {
         'default': {'type': 'virtual', 'skip-install': True},
-        'test': {'type': 'virtual', 'skip-install': True},
+        'test': {'type': 'virtual', 'skip-install': True, 'path': '.venv'},
     }
     assert project.raw_config['tool']['hatch']['envs'] == {
         'default': {'type': 'virtual', 'skip-install': True},
-        'test': {},
+        'test': {'path': '.venv'},
     }
-
-    env_data_path = project_path / '.venvs'
-    assert env_data_path.is_dir()
-
-    storage_dirs = list(env_data_path.iterdir())
-    assert len(storage_dirs) == 1
-
-    storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
-
-    env_dirs = list(storage_path.iterdir())
-    assert len(env_dirs) == 1
-
-    env_path = env_dirs[0]
-
-    assert env_path.name == 'test'
+    assert not env_data_path.is_dir()
+    assert (project_path / '.venv').is_dir()
 
 
 def test_env_var_local_directory(hatch, helpers, temp_dir, config_file):
@@ -378,7 +465,7 @@ def test_env_var_local_directory(hatch, helpers, temp_dir, config_file):
     project = Project(project_path)
     assert project.config.envs == {'default': {'type': 'virtual'}}
     helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
-    helpers.update_project_environment(project, 'test', {})
+    helpers.update_project_environment(project, 'test', {'path': '.foo'})
 
     with project_path.as_cwd({'VENVS_DIR': str(env_data_path), 'HATCH_ENV_TYPE_VIRTUAL_PATH': '.venv'}):
         result = hatch('env', 'create', 'test')
@@ -394,11 +481,11 @@ def test_env_var_local_directory(hatch, helpers, temp_dir, config_file):
     project = Project(project_path)
     assert project.config.envs == {
         'default': {'type': 'virtual', 'skip-install': True},
-        'test': {'type': 'virtual', 'skip-install': True},
+        'test': {'type': 'virtual', 'skip-install': True, 'path': '.foo'},
     }
     assert project.raw_config['tool']['hatch']['envs'] == {
         'default': {'type': 'virtual', 'skip-install': True},
-        'test': {},
+        'test': {'path': '.foo'},
     }
     assert not env_data_path.is_dir()
     assert (project_path / '.venv').is_dir()
@@ -443,16 +530,14 @@ def test_enter_project_directory(hatch, config_file, helpers, temp_dir):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -495,16 +580,14 @@ def test_already_created(hatch, config_file, helpers, temp_dir):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -556,16 +639,14 @@ def test_default(hatch, config_file, helpers, temp_dir):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -621,16 +702,14 @@ def test_matrix(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = sorted(storage_path.iterdir(), key=lambda d: d.name)
     assert len(env_dirs) == 2
@@ -788,16 +867,14 @@ def test_incompatible_matrix_partial(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -841,16 +918,14 @@ def test_install_project_default_dev_mode(
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -902,16 +977,14 @@ def test_install_project_no_dev_mode(hatch, helpers, temp_dir, platform, config_
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1132,16 +1205,14 @@ def test_sync_dependencies(hatch, helpers, temp_dir, platform, config_file, extr
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1197,16 +1268,14 @@ def test_features(hatch, helpers, temp_dir, platform, config_file, extract_insta
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
@@ -1224,6 +1293,167 @@ def test_features(hatch, helpers, temp_dir, platform, config_file, extract_insta
         assert requirements[1].lower() == f'-e {str(project_path).lower()}'
 
 
+@pytest.mark.requires_internet
+def test_sync_dynamic_dependencies(hatch, helpers, temp_dir, platform, config_file, extract_installed_requirements):
+    config_file.model.template.plugins['default']['tests'] = False
+    config_file.save()
+
+    project_name = 'My.App'
+
+    with temp_dir.as_cwd():
+        result = hatch('new', project_name)
+
+    assert result.exit_code == 0, result.output
+
+    for i in range(2):
+        with temp_dir.as_cwd():
+            result = hatch('new', f'{project_name}{i}')
+
+        assert result.exit_code == 0, result.output
+
+    project_path = temp_dir / 'my-app'
+    data_path = temp_dir / 'data'
+    data_path.mkdir()
+
+    project = Project(project_path)
+    config = dict(project.raw_config)
+    config['project'].pop('dependencies')
+    config['project']['dynamic'].extend(('dependencies', 'optional-dependencies'))
+    config['tool']['hatch']['metadata'] = {'allow-direct-references': True, 'hooks': {'custom': {}}}
+    project.save_config(config)
+    helpers.update_project_environment(
+        project,
+        'default',
+        {
+            'dependencies': ['my-app1 @ {root:uri}/../my-app1'],
+            'features': ['foo'],
+            'post-install-commands': ["python -c \"with open('test.txt', 'w') as f: f.write('content')\""],
+            **project.config.envs['default'],
+        },
+    )
+    helpers.update_project_environment(project, 'test', {})
+
+    build_script = project_path / DEFAULT_BUILD_SCRIPT
+    build_script.write_text(
+        helpers.dedent(
+            """
+            from hatchling.metadata.plugin.interface import MetadataHookInterface
+
+            class CustomHook(MetadataHookInterface):
+                def update(self, metadata):
+                    metadata['dependencies'] = ['my-app0 @ {root:uri}/../my-app0']
+                    metadata['optional-dependencies'] = {'foo': ['binary']}
+            """
+        )
+    )
+
+    with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
+        result = hatch('env', 'create', 'test')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Creating environment: test
+        Installing project in development mode
+        Running post-installation commands
+        Checking dependencies
+        Syncing dependencies
+        """
+    )
+    assert (project_path / 'test.txt').is_file()
+
+    env_data_path = data_path / 'env' / 'virtual'
+    assert env_data_path.is_dir()
+
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
+    assert len(storage_dirs) == 1
+
+    storage_path = storage_dirs[0]
+    assert len(storage_path.name) == 8
+
+    env_dirs = list(storage_path.iterdir())
+    assert len(env_dirs) == 1
+
+    env_path = env_dirs[0]
+
+    assert env_path.name == 'test'
+
+    with VirtualEnv(env_path, platform):
+        output = platform.run_command(['pip', 'freeze'], check=True, capture_output=True).stdout.decode('utf-8')
+        requirements = extract_installed_requirements(output.splitlines())
+
+        assert len(requirements) == 4
+        assert requirements[0].startswith('binary==')
+        assert requirements[1].lower() == f'-e {str(project_path).lower()}'
+        assert requirements[2].lower() == f'my-app0 @ {path_to_uri(project_path).lower()}/../my-app0'
+        assert requirements[3].lower() == f'my-app1 @ {path_to_uri(project_path).lower()}/../my-app1'
+
+
+@pytest.mark.requires_internet
+def test_unknown_dynamic_feature(hatch, helpers, temp_dir, platform, config_file, extract_installed_requirements):
+    config_file.model.template.plugins['default']['tests'] = False
+    config_file.save()
+
+    project_name = 'My.App'
+
+    with temp_dir.as_cwd():
+        result = hatch('new', project_name)
+
+    assert result.exit_code == 0, result.output
+
+    with temp_dir.as_cwd():
+        result = hatch('new', f'{project_name}1')
+
+    assert result.exit_code == 0, result.output
+
+    project_path = temp_dir / 'my-app'
+    data_path = temp_dir / 'data'
+    data_path.mkdir()
+
+    project = Project(project_path)
+    config = dict(project.raw_config)
+    config['build-system']['requires'].append(f'my-app1 @ {path_to_uri(project_path).lower()}/../my-app1')
+    config['project']['dynamic'].append('optional-dependencies')
+    config['tool']['hatch']['metadata'] = {'hooks': {'custom': {}}}
+    project.save_config(config)
+    helpers.update_project_environment(
+        project,
+        'default',
+        {
+            'features': ['foo'],
+            'post-install-commands': ["python -c \"with open('test.txt', 'w') as f: f.write('content')\""],
+            **project.config.envs['default'],
+        },
+    )
+    helpers.update_project_environment(project, 'test', {})
+
+    build_script = project_path / DEFAULT_BUILD_SCRIPT
+    build_script.write_text(
+        helpers.dedent(
+            """
+            from hatchling.metadata.plugin.interface import MetadataHookInterface
+
+            class CustomHook(MetadataHookInterface):
+                def update(self, metadata):
+                    metadata['optional-dependencies'] = {'bar': ['binary']}
+            """
+        )
+    )
+
+    with project_path.as_cwd(env_vars={ConfigEnvVars.DATA: str(data_path)}):
+        with pytest.raises(
+            ValueError,
+            match=(
+                'Feature `foo` of field `tool.hatch.envs.test.features` is not defined in the dynamic '
+                'field `project.optional-dependencies`'
+            ),
+        ):
+            hatch('env', 'create', 'test')
+
+
 def test_no_project_file(hatch, helpers, temp_dir, config_file):
     config_file.model.template.plugins['default']['tests'] = False
     config_file.save()
@@ -1255,16 +1485,14 @@ def test_no_project_file(hatch, helpers, temp_dir, config_file):
     env_data_path = data_path / 'env' / 'virtual'
     assert env_data_path.is_dir()
 
-    storage_dirs = list(env_data_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
