@@ -68,19 +68,17 @@ def test_single(hatch, helpers, temp_dir_data, config_file):
 
     assert result.exit_code == 0, result.output
 
-    env_cache_path = temp_dir_data / 'data' / 'env' / 'virtual'
-    assert env_cache_path.is_dir()
+    env_data_path = temp_dir_data / 'data' / 'env' / 'virtual'
+    assert env_data_path.is_dir()
 
-    storage_dirs = list(env_cache_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 2
@@ -131,19 +129,17 @@ def test_all(hatch, helpers, temp_dir_data, config_file):
 
     assert result.exit_code == 0, result.output
 
-    env_cache_path = temp_dir_data / 'data' / 'env' / 'virtual'
-    assert env_cache_path.is_dir()
+    env_data_path = temp_dir_data / 'data' / 'env' / 'virtual'
+    assert env_data_path.is_dir()
 
-    storage_dirs = list(env_cache_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 2
@@ -197,19 +193,17 @@ def test_matrix_all(hatch, helpers, temp_dir_data, config_file):
 
     assert result.exit_code == 0, result.output
 
-    env_cache_path = temp_dir_data / 'data' / 'env' / 'virtual'
-    assert env_cache_path.is_dir()
+    env_data_path = temp_dir_data / 'data' / 'env' / 'virtual'
+    assert env_data_path.is_dir()
 
-    storage_dirs = list(env_cache_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 2
@@ -234,6 +228,51 @@ def test_matrix_all(hatch, helpers, temp_dir_data, config_file):
     assert not storage_path.is_dir()
 
 
+def test_matrix_all_local_directory(hatch, helpers, temp_dir_data, config_file):
+    config_file.model.template.plugins['default']['tests'] = False
+    config_file.model.dirs.env = {'virtual': '.hatch'}
+    config_file.save()
+
+    project_name = 'My.App'
+
+    with temp_dir_data.as_cwd():
+        result = hatch('new', project_name)
+
+    assert result.exit_code == 0, result.output
+
+    project_path = temp_dir_data / 'my-app'
+
+    project = Project(project_path)
+    helpers.update_project_environment(project, 'default', {'skip-install': True, **project.config.envs['default']})
+    helpers.update_project_environment(project, 'foo', {'matrix': [{'version': ['9000', '42']}]})
+
+    with project_path.as_cwd():
+        result = hatch('env', 'create', 'foo')
+
+    assert result.exit_code == 0, result.output
+
+    env_data_path = project_path / '.hatch'
+    assert env_data_path.is_dir()
+
+    env_dirs = list(env_data_path.iterdir())
+    assert len(env_dirs) == 3
+
+    assert sorted(entry.name for entry in env_dirs) == ['.gitignore', 'foo.42', 'foo.9000']
+
+    with project_path.as_cwd():
+        result = hatch('env', 'remove', 'foo')
+
+    assert result.exit_code == 0, result.output
+    assert result.output == helpers.dedent(
+        """
+        Removing environment: foo.9000
+        Removing environment: foo.42
+        """
+    )
+
+    assert not env_data_path.is_dir()
+
+
 def test_incompatible_ok(hatch, helpers, temp_dir_data, config_file):
     project_name = 'My.App'
 
@@ -297,19 +336,17 @@ def test_active_override(hatch, helpers, temp_dir_data, config_file):
 
     assert result.exit_code == 0, result.output
 
-    env_cache_path = temp_dir_data / 'data' / 'env' / 'virtual'
-    assert env_cache_path.is_dir()
+    env_data_path = temp_dir_data / 'data' / 'env' / 'virtual'
+    assert env_data_path.is_dir()
 
-    storage_dirs = list(env_cache_path.iterdir())
+    project_data_path = env_data_path / project_path.name
+    assert project_data_path.is_dir()
+
+    storage_dirs = list(project_data_path.iterdir())
     assert len(storage_dirs) == 1
 
     storage_path = storage_dirs[0]
-
-    project_part = f'{project_path.name}-'
-    assert storage_path.name.startswith(project_part)
-
-    hash_part = storage_path.name[len(project_part) :]
-    assert len(hash_part) == 8
+    assert len(storage_path.name) == 8
 
     env_dirs = list(storage_path.iterdir())
     assert len(env_dirs) == 1
