@@ -629,6 +629,25 @@ class TestDependencies:
 
         assert environment.dependencies == ['dep2', 'dep3']
 
+    def test_unknown_dynamic_feature(self, isolation, data_dir, platform):
+        config = {
+            'project': {'name': 'my_app', 'version': '0.0.1', 'dynamic': ['optional-dependencies']},
+            'tool': {'hatch': {'envs': {'default': {'skip-install': False, 'features': ['foo']}}}},
+        }
+        project = Project(isolation, config=config)
+        environment = MockEnvironment(
+            isolation, project.metadata, 'default', project.config.envs['default'], {}, data_dir, platform, 0
+        )
+
+        with pytest.raises(
+            ValueError,
+            match=(
+                'Feature `foo` of field `tool.hatch.envs.default.features` is not defined in the dynamic '
+                'field `project.optional-dependencies`'
+            ),
+        ):
+            _ = environment.dependencies
+
 
 class TestScripts:
     def test_not_table(self, isolation, data_dir, platform):
@@ -1140,3 +1159,26 @@ class TestContextFormatting:
         )
 
         assert environment.dependencies == ['pkg==42']
+
+    def test_env_vars_override(self, isolation, data_dir, platform):
+        config = {
+            'project': {'name': 'my_app', 'version': '0.0.1'},
+            'tool': {
+                'hatch': {
+                    'envs': {
+                        'default': {
+                            'dependencies': ['pkg{env:DEP_PIN}'],
+                            'env-vars': {'DEP_PIN': '==0.0.1'},
+                            'overrides': {'env': {'DEP_ANY': {'env-vars': 'DEP_PIN='}}},
+                        },
+                    },
+                },
+            },
+        }
+        with EnvVars({'DEP_ANY': 'true'}):
+            project = Project(isolation, config=config)
+            environment = MockEnvironment(
+                isolation, project.metadata, 'default', project.config.envs['default'], {}, data_dir, platform, 0
+            )
+
+            assert environment.dependencies == ['pkg']
